<?php

define('GSALES_REVISION', 1161);

class GS_INSTALLER {

	var $conn;
	var $messages;
	var $content;
	var $mode;
	var $aktuelleRevision;

	var $writeable=array('../DATA/cache_smarty',
						 '../DATA/contract',
						 '../DATA/documents',
						 '../DATA/invoice',
						 '../DATA/offer',
						 '../DATA/refund',
					 );
	
	// typ = 1 => grün
	// typ = 2 => warning aber trotzdem weitermachen (orange)
	// typ = 3 => error (rot)
	
	// benötigte extension
	var $extension=array(
			array('ext' => 'zip','typ' => 2, 'notice' => 'wird benötigt um ZIP-Archive aus mehreren PDF-Dateien (z.B. Rechnungen) zu erstellen'),
			array('ext' => 'soap','typ' => 2, 'notice' => 'wird für die mediafinanz Schnittstelle benötigt'),
			array('ext' => 'curl','typ' => 2, 'notice' => 'wird für die Sicherheitsprüfung des DATA Verzeichnisses benötigt'),
			array('ext' => 'openssl','typ' => 2, 'notice' => 'wird für die mite Schnittstelle benötigt'),
			array('ext' => 'mysqli','typ' => 3)
			//Zend Guard Loader oder Zend Optimizer werden beim PHP Check hinzugefügt
	);

	function __construct(){

		$prechecks=$this->prechecks(); // prechecks ohne die der Installer nicht läuft
		if ($prechecks) {
			$this->envchecks(); // Enviroment checks ohne die GSALES nicht läuft
			if($this->alleChecksOkay() && isset($_POST['install'])) $this->sqlstuff(); //sql abfeueren
		}
		$this->ausgabe();
	}

	function ausgabe(){
		$messageClasses = array(1 => 'alert-success', 2 => '', 3 => 'alert-error');
		foreach ((array)$this->messages as $message){
			echo '<div class="alert '.$messageClasses[$message['typ']].'">';
				echo '<p>';
					echo $message['message'];
					if (isset($message['notice'])){
						if ($message['notice'] != '') echo ' ('.$message['notice'].')';
					}
				echo '</p>';
			echo '</div>';
		}
		echo $this->content;
	}

	function sqlstuff(){

		require 'sql.php'; // db updates und installation durchführen

		foreach($sql as  $rev => $query){
			if($rev > $this->aktuelleRevision){
				if ($this->conn->multi_query($query)) {
				    do {
				        if ($result = $this->conn->store_result()) $result->free();
				    } while ($this->conn->next_result());
				    $this->messages[]=array('message' =>'SQL Anweisungen der Revision #'.$rev.' ausgeführt','typ' => 1);
				} else {
					$this->messages[]=array('message' =>'SQL Anweisungen der Revision #'.$rev.' konnten nicht ausgeführt werden <br /><span class="errorDetails">'.$this->conn->error.'</span>','typ' => 3);
				}
			}
		}

		if($this->mode == 'install'){
			// create GSALES admin user
			$d=$this->conn->query("INSERT INTO user (login,fullname) VALUES ('admin','Administrator')");
			$id=$this->conn->insert_id;
			if($d == false) $this->messages[]=array( 'message' => 'Beim Anlegen des Admin-Benutzers gab es einen Fehler','typ' => 3);

			$pass_enc=$this->randompass();
			$pass=md5($id.'-'.$pass_enc);
			$this->conn->query("UPDATE user SET password='$pass' WHERE id=$id");

			// user admin rolle zuweisen
			$this->conn->query("INSERT INTO rbac_user_roles (user,roles) VALUES ($id,1)");
			
			// set revision
			$d=$this->conn->query("INSERT INTO configuration (id,value) VALUES ('GSALES_REVISION','".GSALES_REVISION."')");
			if($d == false) $this->messages[]=array( 'message' => 'Beim Setzen der Datenbank-Revision (#'. GSALES_REVISION.') gab es einen Fehler','typ' => 3);
			else $this->messages[]=array( 'message' => 'Datenbank-Revision gesetzt (#'. GSALES_REVISION.')','typ' => 1);

			// set magicstring
			$d=$this->conn->query("UPDATE configuration SET value='".$this->randompass(30)."' WHERE id='magicstring'");
			if($d == false) $this->messages[]=array( 'message' => 'Beim Anlegen des Magicstrings gab es einen Fehler','typ' => 3);

			$this->content .= '<br />';
			$this->content .= '<h1>Installation erfolgreich</h1>';
			$this->content .= '<table class="accessData">';
			$this->content .= '	<tr>';
			$this->content .= '		<td><big>Benutzer:</big><big class="code">admin</big></td>';
			$this->content .= '		<td><big>Passwort:</big><big class="code">'.$pass_enc.'</big><span class="warning bold">(unbedingt notieren!)</span></td>';
			$this->content .= '		<td class="right"><a href="../" class="btn btn-small btn-info">zum GSALES-Login</a></td>';
			$this->content .= '	</tr>';
			$this->content .= '</table>';
		}

		if($this->mode == 'update'){
			$d=$this->conn->query("UPDATE configuration SET VALUE='".GSALES_REVISION."' WHERE id='GSALES_REVISION'");
			$this->deleteSmartyCache();
			if($d == false) $this->messages[]=array( 'message' => 'Fehler beim Aktualisieren der Datenbank-Revision (#'. GSALES_REVISION.')','typ' => 3);
			else $this->messages[]=array( 'message' => 'Datenbank-Revision aktualisiert (#'. GSALES_REVISION.')','typ' => 1);
			$this->content .= '<br />';
			$this->content .= '<h1>Update durchgeführt</h1>';
			$this->content .= '<p><a href="../" class="btn btn-small btn-info">zurück zu GSALES</a></p>';
		}
	}

	function envchecks(){
		//writeable
		foreach($this->writeable as $file){
			if ( is_writeable($file)) $this->messages[]=array( 'message' => $file.' beschreibbar','typ' => 1);
			else $this->messages[]=array( 'message' => $file.' ist nicht beschreibbar','typ' => 3);
		}
		//checken ob extension geladen sind
		foreach($this->extension as $val){
			if ( extension_loaded($val['ext'])) $this->messages[]=array( 'message' =>'Erweiterung "'. $val['ext'].'" geladen','typ' => 1);
			else $this->messages[]=array( 'message' =>'Erweiterung "'. $val['ext'].'" nicht geladen','typ' => $val['typ'],'notice' => $val['notice']);
		}
	}

	function prechecks(){

		if(false == is_writeable('../lib/inc.cfg.php') && false == $this->db() ){
			$this->messages[]=array( 'message' => 'Die Config-Datei muss existieren und beschreibbar sein.<br />Bitte die Datei /lib/inc.cfg.dist.php in inc.cfg.php umbenennen und beschreibbar machen.','typ' => 3);
			return FALSE;
		}

		if(false == extension_loaded('mysqli')){
			$this->messages[]=array( 'message' => 'MySQLi Erweiterung muss geladen sein','typ' => 3);
			return FALSE;
		}

		if(false == $this->db()){
			$this->content .= $this->getCfgForm();
			return FALSE;
		}

		$res = $this->conn->query("SELECT value FROM configuration WHERE id='GSALES_REVISION'");
		if (is_object($res)) {
			$data = $res->fetch_assoc();
			$res->close();
		}

		if ($data['value'] == GSALES_REVISION) {
			$this->messages[]=array( 'message' => 'Keine Aktion erforderlich','typ' => 3);
			return FALSE;
		} elseif ($data['value'] == ''){
			$this->mode='install';
			$this->aktuelleRevision=0;
			$this->messages[]=array( 'message' => '<em style="color:#000;">Modus: Installation</em>','typ' => 1);
		} elseif (intval($data['value']) < GSALES_REVISION  ){
			$this->mode='update';
			$this->aktuelleRevision=intval($data['value']);
			$this->messages[]=array( 'message' => '<em style="color:#000;">Modus: Update</em>','typ' => 1);
		} else {
			$this->messages[]=array( 'message' => 'Aktuelle Revision ungültig, bitte melde dich bei unserem Support','typ' => 3);
			return FALSE;
		}

		/*
		if (version_compare(phpversion(), '5.2.0', '<')){
			$this->messages[]=array( 'message' => 'PHP Version ('.phpversion().') nicht kompatibel, benötigt wird 5.2.0 oder neuer!','typ' => 3);
			return FALSE;
		} else {
			$this->messages[]=array( 'message' => 'PHP Version ('.phpversion().')','typ' => 1);
			if(version_compare(phpversion(), '5.3.0', '<'))$this->extension[]=array('ext' => 'Zend Optimizer','typ' => 3, 'notice' => 'wird für die Ausführung der encodierten PHP-Dateien benötigt');
			else $this->extension[]=array('ext' => 'Zend Guard Loader','typ' => 3, 'notice' => 'wird für die Ausführung der encodierten PHP-Dateien benötigt');
		}
		*/

		$res=$this->conn->query("SELECT version() as version");
		if(is_object($res)){
			$data=$res->fetch_assoc();
			$mysql_version=$data['version'];
			$res->close();
		}
		
		if (version_compare($mysql_version, '5.0.0', '<')){
			$this->messages[]=array( 'message' => 'MySQL Version ('.$mysql_version.') nicht kompatibel, benötigt wird 5.0.0 oder neuer!','typ' => 3);
			return FALSE;
		} else {
			$this->messages[]=array( 'message' => 'MySQL Version ('.$mysql_version.')','typ' => 1);
		}

		$mem_limit=$this->getBytes(ini_get("memory_limit"));
		
		if($mem_limit == 0){
			$this->messages[]=array( 'message' => 'php.ini: memory_limit konnte nicht ermittelt werden!','typ' => 2,'notice' => 'Es wird mindestens 32MB Speicher benötigt. Empfohlen werden 64MB');
		} elseif ($mem_limit < (32*1024*1024) ){
			$this->messages[]=array( 'message' => 'php.ini: memory_limit '.($mem_limit/1024/1024).'MB sind nicht ausreichend!','typ' => 3,'notice' => 'Es wird mindestens 32MB Speicher benötigt. Empfohlen werden 64MB');
			return FALSE;
		} elseif ($mem_limit < (64*1024*1024)){
			$this->messages[]=array( 'message' => 'php.ini: memory_limit '.($mem_limit/1024/1024).'MB sind unter Umständen nicht ausreichend!','typ' => 2,'notice' => '64MB Speicher werden empfohlen!');
		} else {
			$this->messages[]=array( 'message' => 'php.ini: memory_limit '.($mem_limit/1024/1024).'MB sind ausreichend!','typ' => 1);
		}

		return TRUE;
	}


	function alleChecksOkay(){

		//wenn noch keine db conn vorhanden brauch man auch nix checken
		if(!$this->db()) return FALSE;
		
		foreach ((array)$this->messages as $val){
			if($val['typ'] == 3){
				$this->content .= $this->getWeiterForm(FALSE);
				return FALSE;
			}
		}

		foreach ((array)$this->messages as $val){
			if($val['typ'] == 2){
				if(!isset($_POST['install']))$this->content .= $this->getWeiterForm('Warnungen ignorieren und Installation starten >>');
				return TRUE;
			}
		}

		if(false == isset($_POST['install'])){
			if($this->mode == 'update') $this->content .= $this->getWeiterForm('Update starten >>');
			if($this->mode == 'install') $this->content .= $this->getWeiterForm('Installation starten >>');
		}
		return true;
	}


	function  db(){

		if(is_writeable('../lib/inc.cfg.php')) {

			if(isset($_POST['check'])){

				if (false == isset($_POST['dbserver'])) return false;
				if (false == isset($_POST['dbuser'])) return false;
				if (false == isset($_POST['dbname'])) return false;
				if (false == isset($_POST['confirmlicenseterms'])) return false;

				# no check for password because it could be emty
				if (trim($_POST['dbserver']) == '') return false;
				if (trim($_POST['dbuser']) == '') return false;
				if (trim($_POST['dbname']) == '') return false;
				if ($_POST['confirmlicenseterms'] != 'on') return false;
				
				$dbserver=$_POST['dbserver'];
				$dbconfig=$this->parseDbServer($dbserver);
				if(!$dbconfig) return false;

				$dbuser=$_POST['dbuser'];
				$dbname=$_POST['dbname'];
				$dbpass=$_POST['dbpass'];	
				$this->conn = @new mysqli($dbconfig['host'], $dbuser, $dbpass, $dbname,$dbconfig['port'],$dbconfig['socket']);
				
				if (0 == mysqli_connect_errno()){
					$this->writecfg();
					return true;
				}
				
			}
			
		}

		if( file_exists('../lib/inc.cfg.php') ) require '../lib/inc.cfg.php';

		if (false == isset($db)) return false;
		
		$dbserver=$db[0]['host'];
		$dbconfig=$this->parseDbServer($dbserver);
		
		$dbuser=$db[0]['user'];
		$dbname=$db[0]['database'];
		$dbpass=$db[0]['password'];

		$this->conn = @new mysqli($dbconfig['host'], $dbuser, $dbpass, $dbname,$dbconfig['port'],$dbconfig['socket']);
		
		$this->conn->query("SET NAMES utf8");
		$this->conn->query("SET CHARACTER SET utf8");
	
		if (0 != mysqli_connect_errno()) return false;
		return true;

	}

	function writecfg(){

		$dbserver=$_POST['dbserver'];
		$dbuser=$_POST['dbuser'];
		$dbname=$_POST['dbname'];
		$dbpass=$_POST['dbpass'];
		
		$inhalt = "<?php\n";
		$inhalt .= "\$db = array();\n";
		$inhalt .= "\$db[0]['host'] = '$dbserver';\n";
		$inhalt .= "\$db[0]['user'] = '$dbuser';\n";
		$inhalt .= "\$db[0]['password'] = '$dbpass';\n";
		$inhalt .= "\$db[0]['database'] = '$dbname';\n";
		$inhalt .= "define('_DATABASECON', serialize(\$db));\n";

		$handle = fopen ('../lib/inc.cfg.php', 'w');

		fwrite ($handle, $inhalt);
		fclose ($handle);
	}

	function getCfgForm(){


		if(isset($_POST['check'])){
			if (isset($_POST['confirmlicenseterms'])){
				if ($_POST['confirmlicenseterms'] != 'on') $this->messages[]=array('message' => 'Bitte Lizenzbedingungen akzeptieren!','typ' => 3);
				else  $this->messages[]=array('message' => 'Datenbankverbindung nicht möglich, bitte die eingegebenen Daten überprüfen.','typ' => 3);
			}
		}

		$strFormDefaults_dbserver = '';
		$strFormDefaults_dbname = '';
		$strFormDefaults_dbuser = '';
		$strFormDefaults_dbpass = '';

		if (isset($_POST['dbserver'])) $strFormDefaults_dbserver = $_POST['dbserver'];
		if (isset($_POST['dbname'])) $strFormDefaults_dbname = $_POST['dbname'];
		if (isset($_POST['dbuser'])) $strFormDefaults_dbuser = $_POST['dbuser'];
		if (isset($_POST['dbpass'])) $strFormDefaults_dbpass = $_POST['dbpass'];

		$ret = '<form method="post">';
		
			$ret .= '<table class="stdForm">';
	
				$ret .= '<tr>';
				$ret .= '	<td class="headline" colspan="4"><h3>Datenbankverbindung</h3></td>';
				$ret .= '</tr>';
				
				$ret .= '<tr class="row1">';
				$ret .= '	<td><label for="dbserver">Server:</label></td>';
				$ret .= '	<td><input type="text" name="dbserver" id="dbserver" value="'.$strFormDefaults_dbserver.'" /></td>';
				$ret .= '	<td><label for="dbname">Datenbank:</label></td>';
				$ret .= '	<td><input type="text" name="dbname" id="dbname" value="'.$strFormDefaults_dbname.'" /><br/>';
				$ret .= '</tr>';

				$ret .= '<tr>';
				$ret .= '	<td><label for="dbuser">Benutzer:</label></td>';
				$ret .= '	<td><input type="text" name="dbuser" id="dbuser" value="'.$strFormDefaults_dbuser.'" /><br/>';
				$ret .= '	<td><label for="dbpass">Passwort:</label></td>';
				$ret .= '	<td><input type="password" name="dbpass" id="dbpass" value="'.$strFormDefaults_dbpass.'" /><br/>';
				$ret .= '</tr>';
				
				$ret .= '<tr>';
				$ret .= '	<td class="headline" colspan="4"><h3>Lizenzbedingungen</h3></td>';
				$ret .= '</tr>';
				
				$licenseterms = $this->getLicenseTerms();
				
				if($licenseterms){
				
					$ret .= '<tr class="row1">';
					$ret .= '	<td colspan="4">';
					$ret .= '		<textarea name="licenseterms" id="licenseterms" readonly="readonly">'.$licenseterms.'</textarea>';
					$ret .= '	</td>';
					$ret .= '</tr>';
					
					$ret .= '<tr>';
					$ret .= '	<td colspan="2">';
					$strChecked = '';

					if (isset($_POST['confirmlicenseterms'])){
						if ($_POST['confirmlicenseterms'] == 'on'){
							$strChecked = 'checked';
						}
					}

					$ret .= '		<input type="checkbox" name="confirmlicenseterms" id="confirmlicenseterms" '.$strChecked.' /> ';
					$ret .= '		<label for="confirmlicenseterms">Lizenzbedingungen gelesen und akzeptiert</label>';
					$ret .= '	</td>';
					$ret .= '	<td colspan="2">';
					$ret .= '		<input type="hidden" value="1" name="check" /><input type="submit" value="Datenbankzugang speichern und weiter >>" name="submit" class="floatR" />';
					$ret .= '	</td>';
					$ret .= '</tr>';
					
				} else {
				
					$ret .= '<tr>';
					$ret .= '	<td colspan="4" class="bold">Die GSALES-Lizenzbedingungen (license_terms.txt) wurden nicht gefunden, Installation kann nicht durchgeführt werden!</td>';
					$ret .= '</tr>';
					
				}
				
			$ret .= '</table>';
	
		$ret .= '</form>';

		return $ret;
	}


	function getWeiterForm($value){

		$ret = '';

		if(false == $value){
			$ret .= '<p>';
				$ret .= '<strong>Installation nicht möglich,</strong> bitte zuerst die aufgetretenen Fehler beheben.<br />';
				$ret .= '<form method="post">';
					$ret .= '<input type="submit" value="Überprüfung erneut durchführen" name="refresh" />';
				$ret .= '</form>';
			$ret .= '</p>';
			return $ret;
		}

		$ret .= '<p>';
			$ret .= '<form method="post">';
				$ret .= '<input type="submit" value="'.$value.'" name="install" />';
			$ret .= '</form>';
		$ret .= '</p>';
		return $ret;
		
		$ret = '<br />';
		$ret .= '<form method="post">';
			$ret .= '<input type="submit" value="Überprüfung erneut durchführen" name="refresh" />';
		$ret .= '</form>';

		return $ret;
	}


	function getLicenseTerms(){
		
		if(file_exists('../license_terms.txt')){
			$ret = false;
			$fp = fopen('../license_terms.txt','r');
		    if($fp){
		    	$licenseterms = '';
		    	while(!feof($fp)){
		    		$licenseterms .= fgets($fp);
		    	}
		    	fclose($fp);
		    }
			$ret = $licenseterms;
		} else {
			$ret = false;
		}
		
		return $ret;
	}

	
	function randompass($length=10) {
	    $characters = 'abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMOPQRSTUV0123456789';
	    $string = '';    
	    for ($p = 0; $p < $length; $p++) $string .= $characters[mt_rand(0, strlen($characters))];
	    return $string;
	}

	function getBytes($str) {
		$str = trim($str);
		$last = strtolower(substr($str,-1));
		$ret=(int)$str;
		switch($last) {
			case 'g':
				$ret *= 1024;
			case 'm':
				$ret *= 1024;
			case 'k':
				$ret *= 1024;
		}
		return $ret;
	}

	function parseDbServer($strServer){
		$explode = explode(':', $strServer);
		if(count($explode) == 1){
			$ret['host'] = $explode[0];
			$ret['port'] = 3306;
			return $ret;
		}

		if(count($explode) != 2) return FALSE;
		
		$ret['host'] = $explode[0];
		if(is_numeric($explode[1])) $ret['port'] = $explode[1];
		else $ret['socket'] = $explode[1];

		if($ret['socket'] != "" && $ret['host'] != 'localhost') return FALSE;

		return $ret;
	}

	function deleteSmartyCache(){
		// try to delete the smarty cache
		$strSearchFor = '../DATA/cache_smarty/*.tpl.php';
		$files = glob( $strSearchFor);
		foreach( $files as $file ) @unlink( $file );
	}
	
}