<?php

/**
 *
 * Standard GSALES 2 PDF Dokumente Template ab 11/2020 für mPDF 8
 * MPDF generiert die PDF-Datei anhand von HTML Markup, welches durch ein SMARTY Template (tpl.def_invoice.tpl) bereitgestellt wird. Siehe unten: $smartyTemplate
 *
 * Das bisherhige Template für die Generierung der PDF Dokumente mittels FPDF steht weiterhin zur Verwendung unter tpl.def_invoice_legacy.php zur Verfügung.
 *
 */

// Anhand vom Datei-ort ermitteln ob es sich um das GSALES Standard-Template handelt oder um ein vom Kunden im Template-Editor selbst angelegtest
// GSALES Standard-Template befindet sich in /lib/tpl/out_pdf/ - Kunden-Templates befinden sich in /DATA/
$booCustomTemplate = true;
if (strpos(__FILE__, 'out_pdf')) $booCustomTemplate = false;

if ($booCustomTemplate) require_once __DIR__ . '/../lib/3rdparty/vendor/autoload.php';
else require_once __DIR__ . '/../../3rdparty/vendor/autoload.php';

// GSALES Konfigurationseinstellungen für die Generierung von PDF Dokumenten
$arrPDFConfig = array();

// PDF Hintergrund
$arrPDFConfig['use_stationery_pdf'] = $this->refCore->cfg->v('pdf_invoice_stationary');
$arrPDFConfig['stationery_pdf_file'] = $this->refCore->cfg->v('path_absolute').$this->refCore->cfg->v('pdf_invoice_stationary_file'); # bitte pdf Datei entsprechend hochladen!

// Mini Absender
$arrPDFConfig['print_minisender'] = $this->refCore->cfg->v('pdf_invoice_printminisender');

/*
    Firmenlogo einbinden
	Möglich sind folgende Dateiformate
	- JPG Bilder (Graustufenbilder, Truecolor 24 Bit, CMYK 32 Bit)
	- PNG Bilder (Graustufenbilder 8 Bit & 256 Graustufen, Farbpaletten, Truecolor 24 Bit)
*/
$arrPDFConfig['use_picture'] = $this->refCore->cfg->v('pdf_invoice_picture');
$arrPDFConfig['picture_file'] = $this->refCore->cfg->v('path_absolute').$this->refCore->cfg->v('pdf_invoice_picture_file'); # GSALES Logo als Beispiel, bitte eigene Verwenden damit diese bei Updates nicht versehentlich überschrieben wird
$arrPDFConfig['picture_width'] = $this->refCore->cfg->v('pdf_invoice_picture_width');
$arrPDFConfig['picture_posX'] = $this->refCore->cfg->v('pdf_invoice_picture_posx');
$arrPDFConfig['picture_posY'] = $this->refCore->cfg->v('pdf_invoice_picture_posy');

// Fußzeilenblöcke
$arrPDFConfig['print_footer_blocks'] = $this->refCore->cfg->v('pdf_invoice_printfooter');

// Fußzeilenblöcke im GSALES 1 Stil = Einfache Textfelder aus Konfiguration
$arrPDFConfig['footer_head_1'] = $this->refCore->cfg->v('pdf_footer_headline_1');
$arrPDFConfig['footer_1'] = $this->refCore->cfg->v('pdf_footer_1');
$arrPDFConfig['footer_head_2'] = $this->refCore->cfg->v('pdf_footer_headline_2');
$arrPDFConfig['footer_2'] = $this->refCore->cfg->v('pdf_footer_2');
$arrPDFConfig['footer_head_3'] = $this->refCore->cfg->v('pdf_footer_headline_3');
$arrPDFConfig['footer_3'] = $this->refCore->cfg->v('pdf_footer_3');
$arrPDFConfig['footer_head_4'] = $this->refCore->cfg->v('pdf_footer_headline_4');
$arrPDFConfig['footer_4'] = $this->refCore->cfg->v('pdf_footer_4');

// Fußzeilenblöcke Content (verwendet den Konfigurationsreiter "Meine Daten")
if ($this->refCore->cfg->v('me_ustidnr') != '' && $this->refCore->cfg->v('me_taxno')){
	$arrTaxFooter = array('headline'=>'Steuernummern', 'txt'=>'USt-IdNr. '.$this->refCore->cfg->v('me_ustidnr')."\nSt.-Nr. ".$this->refCore->cfg->v('me_taxno'));
} else {
	if ($this->refCore->cfg->v('me_ustidnr') != '') $arrTaxFooter = array('headline'=>'USt-IdNr.', 'txt'=>$this->refCore->cfg->v('me_ustidnr'));
	else $arrTaxFooter = array('headline'=>'Steuernummer', 'txt'=>$this->refCore->cfg->v('me_taxno'));
}

$arrContactFooter = array('headline'=>'Anschrift', 'txt'=>$this->refCore->cfg->v('me_company')."\n".$this->refCore->cfg->v('me_address')."\n".$this->refCore->cfg->v('me_countrycode').$this->refCore->cfg->v('me_zip').' '.$this->refCore->cfg->v('me_city'));
if ($this->refCore->cfg->v('me_owner') != '' || $this->refCore->cfg->v('me_manager') != '') $arrContactFooter['txt'] .= "\n";
if ($this->refCore->cfg->v('me_owner') != '') $arrContactFooter['txt'] .= "\n".'Inh. '.$this->refCore->cfg->v('me_owner');
if ($this->refCore->cfg->v('me_manager') != '') $arrContactFooter['txt'] .= "\n".'GF '.$this->refCore->cfg->v('me_manager');

$arrBankFooter = array('headline'=>'Bankverbindung', 'txt'=>'');
if ($this->refCore->cfg->v('me_bank') != '') $arrBankFooter['txt'] .= $this->refCore->cfg->v('me_bank')."\n";
if ($this->refCore->cfg->v('me_bankaccount') != '') $arrBankFooter['txt'] .= 'Konto '.$this->refCore->cfg->v('me_bankaccount')."\n";
if ($this->refCore->cfg->v('me_bankcode') != '') $arrBankFooter['txt'] .= 'BLZ '.$this->refCore->cfg->v('me_bankcode')."\n";
if ($this->refCore->cfg->v('me_bankiban') != '') $arrBankFooter['txt'] .= 'IBAN '.$this->refCore->cfg->v('me_bankiban')."\n";
if ($this->refCore->cfg->v('me_bankbic') != '') $arrBankFooter['txt'] .= 'BIC '.$this->refCore->cfg->v('me_bankbic')."\n";

$arrPDFConfig['arrFooter'] = array(
	$arrContactFooter,
	array('headline'=>'Kontakt', 'txt'=>"Tel. ".$this->refCore->cfg->v('me_phone')."\nFax ".$this->refCore->cfg->v('me_fax')."\n".$this->refCore->cfg->v('me_mail')."\n".$this->refCore->cfg->v('me_web')),
	$arrTaxFooter,
	$arrBankFooter
);

// Sonstige Einstellungen
$arrPDFConfig['displayDecimals'] = $this->refCore->cfg->v('decimalplaces_output');
$arrPDFConfig['showTax'] = $this->refCore->cfg->v('pdf_invoice_tax'); // MwSt. anzeigen?) (für Kleinunternehmner)
$arrPDFConfig['hideDiscount'] = $this->refCore->cfg->v('pdf_invoice_hide_discount'); // Rabatt-Spalte anzeigen wenn kein Rabatt gewährt wurde?
$arrPDFConfig['alternateBg'] = $this->refCore->cfg->v('pdf_alternate_bg'); // Alternierender Hintergrund
$arrPDFConfig['show_colpos'] = $this->refCore->cfg->v('pdf_invoice_position');

// Variablen für ZUGFeRD XML
$arrPDFConfig['zug_default_currency_id'] = $this->refCore->cfg->v('currency_zugferd');
$arrPDFConfig['zug_creditor_account_account_name'] = $this->refCore->cfg->v('me_bankinhaber'); // Kontoinhaber
$arrPDFConfig['zug_creditor_ibanid'] = $this->refCore->cfg->v('me_bankiban');
$arrPDFConfig['zug_creditor_fi_bicid'] = $this->refCore->cfg->v('me_bankbic');
$arrPDFConfig['zug_creditor_fi_bank_name'] = $this->refCore->cfg->v('me_bank');
$arrPDFConfig['zug_seller_name'] = $this->refCore->cfg->v('me_company');
$arrPDFConfig['zug_seller_person_name'] = $this->refCore->cfg->v('me_manager');
$arrPDFConfig['zug_seller_person_email'] = $this->refCore->cfg->v('me_mail');
$arrPDFConfig['zug_seller_person_phone'] = $this->refCore->cfg->v('me_phone');
$arrPDFConfig['zug_seller_postcode_code'] = $this->refCore->cfg->v('me_zip');
$arrPDFConfig['zug_seller_line_one'] = $this->refCore->cfg->v('me_address');
$arrPDFConfig['zug_seller_city_name'] = $this->refCore->cfg->v('me_city');
$arrPDFConfig['zug_seller_country_id'] = $this->refCore->cfg->v('me_countrycode');
$arrPDFConfig['zug_seller_tax_id'] = $this->refCore->cfg->v('me_ustidnr');

// Absender (kann auch über die GSALES Konfiguration beinflusst werden)
$arrPDFConfig['company_sender'] = $this->refCore->cfg->v('me_company').' - '.$this->refCore->cfg->v('me_address').' - '.trim($this->refCore->cfg->v('me_countrycode').' '.$this->refCore->cfg->v('me_zip')).' '.$this->refCore->cfg->v('me_city');

// Labels
$arrPDFConfig['label_invoice'] = $this->refCore->cfg->v('pdf_i_label_invoice');
$arrPDFConfig['label_offer'] = $this->refCore->cfg->v('pdf_i_label_offer');
$arrPDFConfig['label_refund'] = $this->refCore->cfg->v('pdf_i_label_refund');
$arrPDFConfig['label_canceled'] = $this->refCore->cfg->v('pdf_i_label_canceled');
$arrPDFConfig['label_date'] = $this->refCore->cfg->v('pdf_i_label_date');
$arrPDFConfig['label_customerno'] = $this->refCore->cfg->v('pdf_i_label_customerno');
$arrPDFConfig['label_invoiceno'] = $this->refCore->cfg->v('pdf_i_label_invoiceno');
$arrPDFConfig['label_deliverydate'] = $this->refCore->cfg->v('pdf_i_label_deliverydate');
$arrPDFConfig['label_offerno'] = $this->refCore->cfg->v('pdf_i_label_offerno');
$arrPDFConfig['label_refundno'] = $this->refCore->cfg->v('pdf_i_label_refundno');
$arrPDFConfig['label_sale'] = $this->refCore->cfg->v('pdf_i_label_sale');
$arrPDFConfig['label_saleno'] = $this->refCore->cfg->v('pdf_i_label_saleno');
$arrPDFConfig['label_delivery'] = $this->refCore->cfg->v('pdf_i_label_delivery');
$arrPDFConfig['label_deliveryno'] = $this->refCore->cfg->v('pdf_i_label_deliveryno');
$arrPDFConfig['label_page'] = $this->refCore->cfg->v('pdf_i_label_page');
$arrPDFConfig['label_position'] = $this->refCore->cfg->v('pdf_i_label_position');
$arrPDFConfig['label_amount'] = $this->refCore->cfg->v('pdf_i_label_amount');
$arrPDFConfig['label_description'] = $this->refCore->cfg->v('pdf_i_label_description');
$arrPDFConfig['label_discount'] = $this->refCore->cfg->v('pdf_i_label_discount');
$arrPDFConfig['label_tax'] = $this->refCore->cfg->v('pdf_i_label_tax');
$arrPDFConfig['label_unitprice'] = $this->refCore->cfg->v('pdf_i_label_unitprice');
$arrPDFConfig['label_amountprice'] = $this->refCore->cfg->v('pdf_i_label_amountprice');
$arrPDFConfig['label_netamount'] = $this->refCore->cfg->v('pdf_i_label_netamount');
$arrPDFConfig['label_includeddiscount'] = $this->refCore->cfg->v('pdf_i_label_includeddiscount');
$arrPDFConfig['label_plustax'] = $this->refCore->cfg->v('pdf_i_label_plustax');
$arrPDFConfig['label_invoicetotal'] = $this->refCore->cfg->v('pdf_i_label_invoicetotal');
$arrPDFConfig['label_refundtotal'] = $this->refCore->cfg->v('pdf_i_label_refundtotal');
$arrPDFConfig['label_offertotal'] = $this->refCore->cfg->v('pdf_i_label_offertotal');





// Im Blanko keine Bilder, Briefpapier und Fußzeilenblöcke ausgeben
if ($booBlanko){
	$arrPDFConfig['use_stationery_pdf'] = false;
	$arrPDFConfig['use_picture'] = false;
	$arrPDFConfig['print_footer_blocks'] = false;
}


// Smarty die Rechnungsdaten und -konfiguration übergeben
$this->refCore->smarty->assign('pdf_config',$arrPDFConfig);
$this->refCore->smarty->assign('pdf_data',$var_array);

$defaultConfig = (new Mpdf\Config\ConfigVariables())->getDefaults();
$fontDirs = $defaultConfig['fontDir'];

$defaultFontConfig = (new Mpdf\Config\FontVariables())->getDefaults();
$fontData = $defaultFontConfig['fontdata'];

$strFontsDir = __DIR__ . '/../../../DATA//mpdf_fonts';
if ($booCustomTemplate) $strFontsDir = __DIR__ . '/../DATA//mpdf_fonts';

// PDF/A nur aktivieren wenn ZUGFeRD Version ausgewählt wurde
// PDF/A bettet u.a. die komplette Schrift ein - dadurch steigt die Dateigröße
$booPDFA = false;
if ($var_array['base']['i_zugferd'] != 'false') $booPDFA = true;

/*
Die Schrift arial.ttf ist standard.
Die Schriften arial-latin-only.ttf und arial-de-only.ttf haben
wir für euch erstellt/angepasst (weniger Zeichen = kleinere Dateigröße)
artial.ttf               -> latin, greek, arabic, usw. -> 1.013KB
artial-latin-only.ttf    -> nur latin                  -> 280KB
artial-de-only.ttf       -> nur deutsche zeichen       -> 197KB
*/

// Wenn PDF dann kann arial.ttf verwndet werden - es werden nur die Zeichen eingebettet die notwendig sind
if (false == $booPDFA) $strFontToUse = 'arial.ttf';

// Wenn PDF/A dann ermitteln ob wird mit arial-de.ttf auskommen - ansonsten arial-latinonly.ttf verwenden
if ($booPDFA){
	$strFontToUse = 'arial-latin-only.ttf';
	$strToTest = $arrPDFConfig['company_sender'].$var_array['base']['recipient_txt'].$var_array['base']['vars_i_pre_txt'].$var_array['base']['vars_i_post_txt'];
	foreach($var_array['pos'] as $pos) $strToTest .= $pos['vars_pos_txt'];
	if (!preg_match('/[^[:ascii:]öäüÖÄÜß€§]/u', $strToTest)) $strFontToUse = 'arial-de-only.ttf';
}

$mpdf = new \Mpdf\Mpdf([

	'fontDir' => array_merge($fontDirs, [
		$strFontsDir
	]),

	'fontdata' => $fontData + [
			'arial' => [
				'R' => $strFontToUse,
			]
		],
	'mode' => 'utf-8',
	'PDFA' => $booPDFA,
	'PDFAauto' => $booPDFA,
]);

// ZUGFeRD XML
if ($var_array['type'] == 'invoices'){

	if ($var_array['base']['i_zugferd'] == 'zf1'){
		// zugferd 1.0
		$mpdf->SetAssociatedFiles([[
			'name' => 'ZUGFeRD-invoice.xml',
			'mime' => 'text/xml',
			'description' => 'ZUGFeRD 1.0',
			'AFRelationship' => 'Alternative',
			'content' => $this->refCore->smarty->fetch($this->refCore->cfg->v('path_absolute').'lib/tpl/out_pdf/tpl.zugferd_1_invoice_basic.tpl')
		]]);
		$rdf  = '<rdf:Description rdf:about="" xmlns:zf="urn:ferd:pdfa:CrossIndustryDocument:invoice:1p0#">'."\n";
		$rdf .= '  <zf:DocumentType>INVOICE</zf:DocumentType>'."\n";
		$rdf .= '  <zf:DocumentFileName>ZUGFeRD-invoice.xml</zf:DocumentFileName>'."\n";
		$rdf .= '  <zf:Version>1.0</zf:Version>'."\n";
		$rdf .= '  <zf:ConformanceLevel>BASIC</zf:ConformanceLevel>'."\n";
		$rdf .= '</rdf:Description>'."\n";
		$mpdf->SetAdditionalXmpRdf($rdf);
	}

	if ($var_array['base']['i_zugferd'] == 'zf211'){
		// zugferd 2.1.1
		$mpdf->SetAssociatedFiles([[
			'name' => 'factur-x.xml',
			'mime' => 'text/xml',
			'description' => 'ZUGFeRD 2.1.1',
			'AFRelationship' => 'Alternative',
			'content' => $this->refCore->smarty->fetch($this->refCore->cfg->v('path_absolute').'lib/tpl/out_pdf/tpl.zugferd_211_invoice_basic.tpl')
		]]);
		$rdf  = '<rdf:Description rdf:about="" xmlns:fx="urn:factur-x:pdfa:CrossIndustryDocument:invoice:1p0#">'."\n";
		$rdf .= '  <fx:ConformanceLevel>BASIC</fx:ConformanceLevel>'."\n";
		$rdf .= '  <fx:DocumentFileName>factur-x.xml</fx:DocumentFileName>'."\n";
		$rdf .= '  <fx:DocumentType>INVOICE</fx:DocumentType>'."\n";
		$rdf .= '  <fx:Version>1.0</fx:Version>'."\n";
		$rdf .= '</rdf:Description>'."\n";
		$mpdf->SetAdditionalXmpRdf($rdf);
	}

}

// Briefpapier im Hintergrund einbinden?
if($arrPDFConfig['use_stationery_pdf']) $mpdf->SetDocTemplate($arrPDFConfig['stationery_pdf_file'], true);

$smartyTemplate = $this->refCore->cfg->v('path_absolute').'lib/tpl/out_pdf/tpl.def_invoice.tpl';

# Für die Verwendung eines eigenen Smarty-Templates folgende Zeile einkommentieren und den Dateinamen anpassen
# $smartyTemplate = $this->refCore->cfg->v('path_absolute').'DATA/my.def_invoice.tpl';

$mpdf->WriteHTML($this->refCore->smarty->fetch($smartyTemplate));

// PDF Datei ausgeben
if ($booBlanko) $mpdf->Output($savetoblanko,'F');
else $mpdf->Output($saveto,'F');

unset($mpdf);